import os
import time
import random
import logging

from sbdk.docker import make_runner, make_client, find_image
from sbdk.executor import Adun
from sbdk.error import SbdkError
from sbgsdk.job import Job
from sbgsdk.protocol import to_json, from_json


def argument_parser(subparsers):
    parser = subparsers.add_parser("cf", help="Run from CF")
    parser.set_defaults(func=run)

    parser.add_argument("--job", help="Path to JSON file with Job", type=str, required=True)
    parser.add_argument("--result", help="Path to future JSON file with result of job", type=str, required=True)
    parser.add_argument("--image-repo", help="Docker image repo", type=str, required=True)
    parser.add_argument("--image-tag", help="Docker image tag", type=str, required=True)


def run(project, job, image_repo, image_tag, result):
    assert project and job and image_tag and image_repo and result, 'All arguments required.'
    job_file_path = job
    with open(job_file_path) as fp:
        job = from_json(fp)
        assert isinstance(job, Job), "Expected Job in %s" % job_file_path

    docker_client = make_client(project)
    image = get_image(docker_client, image_repo, image_tag)
    os.chdir(project.project_dir)
    project.context['image_id'] = image['Id']
    executor = Adun(make_runner(project, Volumes={'/mnt/ebs': {}}, __binds__={'/mnt/ebs': '/mnt/ebs:rw'}))
    r = executor.exec_wrapper_job(job)
    with open(result, 'w') as fp:
        to_json(r, fp)


def get_image(client, repo, tag, retries=5):
    logging.info('Searching for image %s:%s', repo, tag)
    img = find_image(client, repo, tag)
    if img:
        return img
    if retries < 1:
        raise SbdkError('Image not found: %s tag: %s' % (repo, tag))
    client.pull(repo, tag)
    time.sleep(1 + random.random() * 10)  # Weird stuff happening on simultaneous pull + run.
    return get_image(client, repo, tag, retries-1)


import json
import os.path
import requests
from sys import exit

from sbdk.auth import push_url, push_json
from sbdk.commands import check_project_dir, check_image, check_project_data
from sbdk.docker import make_runner


def argument_parser(subparsers):
    parser = subparsers.add_parser("push", help="Create new image and upload it")

    parser.set_defaults(func=push)

    parser.add_argument('message', help='Short description of your changes')


def push(project, message):
    check_project_dir(project)
    check_project_data(project)
    check_image(project)

    docker = make_runner(project)
    container = docker.run_install()
    if not container.is_success():
        print("Error while preparing image:")
        container.print_log()
        exit("Push command failed")
    config = {'Cmd': project.sdk_runner_command, 'Entrypoint': project.docker_entrypoint}
    container.load_credentials(renew=True)
    container.commit(message, config).push()
    push_schema(project, container)
    container.remove()


def push_schema(project, container):
    data_dir = project.project_data
    # nulling project_data to force image runner
    project.project_data = None
    project.image_id = container.image['Id']
    runner = make_runner(project)
    runner.schema('.sbdk/schema.json')

    with open(os.path.join(data_dir, 'schema.json')) as fp:
        wrapper_data = json.load(fp)  # json.loads(self.docker.logs(c))

    if not wrapper_data:
        print('No wrappers registered (empty __init__.py?). Exiting.')
        exit(1)

    push_data = {'message': container.message or 'NO COMMIT MESSAGE',
                 'image_data': {'id': project.docker_registry_name + '/' + container.user + '/' + project.name,
                                'tag': container.tag},
                 'wrappers': wrapper_data}
    if project.schema_server_url:
        url = push_url(project, container.user)
        try:
            push_json(url, push_data, container.token)
        except requests.HTTPError as e:
            if e.response.status_code in (401, 402):
                container.load_credentials(renew=True)
                push_json(url, push_data, container.token)
            else:
                raise e